@@ -93,7 +93,10 @@ int check_header(char *buffer, unsigned int size, unsigned int prior_ts, struct
 
   /* does the buffer already contain a valid packet header (without any correction) ?? */
   memcpy(hdr, buffer, sizeof(struct packet_hdr_s));
-  if (is_plausible(*hdr, prior_ts) == 0) return(0);
+  res = is_plausible(*hdr, prior_ts);
+  if (res == 0) return(0);
+
+  if (verbose >= 2) printf("[-] Header plausibility check failed with error code %d\n", res);
 
   /* we need to abort the recursion of there are too many possible ascii corrupted bytes at ones */
   /* 32-25 = 7 bytes maximum in 32bytes of data! */
@@ -170,6 +173,12 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
 
   /* BEGIN GLOBAL HEADER CHECK */
 
+  /* check space of pcap global header */
+  if (filesize < sizeof(global_hdr)) {
+    printf("[-] File is too small to read pcap global header.\n");
+    return(-2);
+  }
+
   printf("[*] Analyzing Global Header...\n");
   bytes = fread(&global_hdr, sizeof(global_hdr), 1, pcap);	/* read first bytes of input file into struct */
   if (bytes != 1) return -3;
@@ -262,6 +271,7 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
   if (data_link_type != -1) {
     printf("[+] Changing data link type to %d.\n", data_link_type);
     global_hdr.network = conint(data_link_type);
+    hdr_integ++;
   }
 
   /* evaluate the integrity of the global header */
@@ -315,8 +325,25 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
       step++;
     }
 
+    /* check if a pcap packet header would fit into the file */
+    if (pos + sizeof(hdrbuffer) > filesize) {
+      printf("[-] Not enough bytes left in file ==> SKIPPING %" PRIu64 " bytes.\n", filesize-pos);
+      corrupted++;
+
+      /* at least one packet has been found?? */
+      if (count == 1) {
+        printf("[-] No packet has been found!\n");
+        return(-2);
+      }
+
+      /* end of file reached */
+      pos = filesize;
+      break;
+    }
+
     /* read the next packet header */
     bytes = fread(hdrbuffer, sizeof(hdrbuffer), 1, pcap);
+    if (bytes != 1) return -3;
 
     /* check if the packet header looks proper */
     res = check_header(hdrbuffer, sizeof(hdrbuffer), last_correct_ts_sec, &packet_hdr);
@@ -521,7 +548,7 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
                 packet_hdr.ts_usec = conint(0);
               }
             } else { /* ALL packets except the first one will use the last correct packets timestamps */
-              packet_hdr.ts_sec = last_correct_ts_sec;
+              packet_hdr.ts_sec = conint(last_correct_ts_sec);
               packet_hdr.ts_usec = conint(last_correct_ts_usec+1);
             }
 
@@ -529,16 +556,25 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
             fseeko(pcap, pos+16, SEEK_SET);
             bytes = fread(&buffer, conint(packet_hdr.incl_len), 1, pcap);
 
-            /* write repaired packet header and packet body */
-            bytes = fwrite(&packet_hdr, sizeof(packet_hdr), 1, pcap_fix);	/* write packet header to output file */
-            bytes = fwrite(&buffer, conint(packet_hdr.incl_len), 1, pcap_fix);	/* write packet body to output file */
+            /* final check resulting packet for plausibility */
+            res = is_plausible(packet_hdr, last_correct_ts_sec);
+            if (res == 0) {
 
-            /* remember that this packets timestamp to evaluate futher timestamps */
-            last_correct_ts_sec = packet_hdr.ts_sec;
-            last_correct_ts_usec = packet_hdr.ts_usec;
+              /* write repaired packet header and packet body */
+              bytes = fwrite(&packet_hdr, sizeof(packet_hdr), 1, pcap_fix);	/* write packet header to output file */
+              bytes = fwrite(&buffer, conint(packet_hdr.incl_len), 1, pcap_fix);	/* write packet body to output file */
 
-            /* print out information */
-            printf("[+] CORRECTED Packet #%u at position %" PRIu64 " (%u | %u | %u | %u).\n", count, pos, conint(packet_hdr.ts_sec), conint(packet_hdr.ts_usec), conint(packet_hdr.incl_len), conint(packet_hdr.orig_len));
+              /* remember that this packets timestamp to evaluate futher timestamps */
+              last_correct_ts_sec = conint(packet_hdr.ts_sec);
+              last_correct_ts_usec = conint(packet_hdr.ts_usec);
+
+              /* print out information */
+              printf("[+] CORRECTED Packet #%u at position %" PRIu64 " (%u | %u | %u | %u).\n", count, pos, conint(packet_hdr.ts_sec), conint(packet_hdr.ts_usec), conint(packet_hdr.incl_len), conint(packet_hdr.orig_len));
+
+            } else {
+              /* prior packet is invalid */
+              printf("[-] Packet #%u at position %" PRIu64 " is invalid ==> SKIPPING.\n", count, pos);
+            }
 
           }
 
@@ -563,6 +599,14 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
         break;
       }
 
+      /* maximum search range reached? -> skip packet and keep on searching */
+      if (deep_scan == 0 && (nextpos > pos+16+65535)) {
+        if (verbose >= 1) printf("[-] No next packet found within max packet range --> SKIPPING!\n");
+
+        /* reset counter because no packet found */
+        count--;
+      }
+
     }
 
     /* get current file pointer position to start next loop iteration */
