@@ -21,8 +21,11 @@
 #define PF_PCAP
 
 #define PCAP_MAGIC 0xa1b2c3d4			/* the magic of the pcap global header (non swapped) */
+#define PCAP_MAGIC_SWAPPED 0xd4c3b2a1           /* the magic of the pcap global header (swapped) */
 #define PCAPNG_MAGIC 0x0a0d0d0a			/* the magic of the pcap global header (non swapped) */
 #define PCAP_NSEC_MAGIC 0xa1b23c4d	    /* the magic of the pcap global header (nanoseconds - non swapped) */
+#define PCAP_EXT_MAGIC 0xa1b2cd34               /* the magic of the extended pcap global header (non swapped) */
+#define PCAP_EXT_MAGIC_SWAPPED 0x34cdb2a1       /* the magic of the extended pcap global header (swapped) */
 
 /* Global header (http://v2.nat32.com/pcap.htm) */
 struct global_hdr_s {
@@ -43,6 +46,15 @@ struct packet_hdr_s {
   u_int32_t orig_len;       /* actual length of packet */
 };
 
+/* Extended Packet Header - UNUSED YET
+   http://tcpreplay.synfin.net/doxygen_yhsiam/tcpcapinfo_8c-source.html - is there an official documentation? */
+struct ext_packet_hdr_s {
+  struct packet_hdr_s ph;   /* the default pcap packet header */
+  int32_t index;	    /* some index */
+  u_int16_t protocol;       /* some protocol */
+  u_int8_t pkt_type;        /* some packet type */
+};
+
 /*
  * Function:  is_plausible
  * -----------------------
