@@ -15,28 +15,28 @@
  * You should have received a copy of the GNU General Public License along with
  * Pcapfix. If not, see http://www.gnu.org/licenses/.
  *
- * Last Modified: 18.02.2014
+ * Last Modified: 31.08.2014
  *
  *******************************************************************************
  *
  * Description:
  *
- * pcapfix is a repair tool for corrupted pcap/pcapng files. Depending on the
- * format, it ...
+ * Pcapfix is a tool to repair your damaged or corrupted pcap and pcapng files.
+ * It is written in C and released under the GNU General Public License.
  *
- * (pcap format) ...checks for an intact pcap global header and repairs it if
- * there are any corrupted bytes (pcap format). If one is not present, one is
- * created and added to the beginning of the file. It then tries to find pcap
- * packet headers, and checks and repairs them. pcapfix will first step through
- * the packets top down until it recognizes a corrupted one by using plausibility
- * checks. After that the tool will brute force further pcap packet headers by
- * reading the file byte by byte. If another proper packet is found, pcapfix
- * restores the data in between by adding a well-formed pcap packet header.
+ * To fix your pcap files the tool first checks for an intact pcap global header
+ * and repairs it if there are some corrupted bytes. It there seems to be no
+ * global header at all, pcapfix adds a self-created one at the beginning of the
+ * file. In a second step the tool tries to find pcap packet headers inside the
+ * file, below the global header. It checks if the values are correct (or seem
+ * to be correct) and tries to repair a packet if there is something wrong. 
  *
- * (pcapng format) ...loops the block heades one by one checking for section
- * header block and interface description block and creates one if missing. It
- * progresses all blocks checking the options / records, skips invalid ones
- * and aligns next header if there is any corruption.
+ * To fix your pcapng files the tool loops through all packet headers that can
+ * be found in the file. It checks for mandatory Section Header Block and
+ * Interface Description Block and creates them if missing. Pcapfix checks for
+ * correct block sizes and valid option fields. If something is wrong, invalid
+ * fields are repaired (if possible) or skipped and adjusted to finally get a
+ * proper pcapng file.
  *
  ******************************************************************************/
 
@@ -44,8 +44,9 @@
 #include "pcap.h"
 #include "pcapng.h"
 
-#define VERSION "1.0.2"			    /* pcapfix version */
+#define VERSION "1.1.0"			    /* pcapfix version */
 
+#define BTSNOOP_MAGIC 0x6E737462    /* btsnoop file magic (first 4 bytes) */
 #define SNOOP_MAGIC 0x6f6f6e73	    /* snoop file magic (first 4 bytes) */
 #define NETMON_MAGIC 0x55424d47     /* netmon file magic */
 #define NETMON11_MAGIC 0x53535452   /* netmon 1.1 file magic */
@@ -73,10 +74,11 @@ unsigned int header_magic;
 void usage(char *progname) {
   printf("Usage: %s [OPTIONS] filename\n", progname);
   printf("OPTIONS:");
-  printf(  "\t-d     , --deep-scan          \tDeep scan (pcap only)\n");
-  printf("\t\t-n     , --pcapng             \tforce pcapng format\n");
-  printf("\t\t-t <nr>, --data-link-type <nr>\tData link type\n");
-  printf("\t\t-v     , --verbose            \tVerbose output\n");
+  printf(  "\t-d        , --deep-scan          \tDeep scan (pcap only)\n");
+  printf("\t\t-n        , --pcapng             \tforce pcapng format\n");
+  printf("\t\t-o <file> , --outfile <file>     \tset output file name\n");
+  printf("\t\t-t <nr>   , --data-link-type <nr>\tData link type\n");
+  printf("\t\t-v        , --verbose            \tVerbose output\n");
   printf("\n");
 }
 
@@ -168,7 +170,7 @@ int main(int argc, char *argv[]) {
   int res;                      /* return values */
   char *filename;               /* filename of input file */
   char *filebname;              /* filebasename of input file (without path) */
-  char *filename_fix;           /* filename of output file */
+  char *filename_fix = NULL;    /* filename of output file */
   uint64_t bytes;		        /* read/written blocks counter */
   uint64_t filesize;	        /* file size of input pcap file in bytes */
 
@@ -176,6 +178,7 @@ int main(int argc, char *argv[]) {
   struct option long_options[] = {
     {"deep-scan", no_argument, 0, 'd'},				        /* --deep-scan == -d */
     {"pcapng", no_argument, 0, 'n'},				        /* --pcapng == -n */
+    {"outfile", required_argument, 0, 'o'},		    /* --outfile == -o */
     {"data-link-type", required_argument, 0, 't'},		    /* --data-link-type == -t */
     {"verbose", no_argument, 0, 'v'},				        /* --verbose == -v */
     {0, 0, 0, 0}
@@ -185,7 +188,7 @@ int main(int argc, char *argv[]) {
   printf("pcapfix %s (c) 2012-2014 Robert Krause\n\n", VERSION);
 
   /* scan for options and arguments */
-  while ((c = getopt_long(argc, argv, ":t:v::d::n::", long_options, &option_index)) != -1) {
+  while ((c = getopt_long(argc, argv, ":t:o:v::d::n::", long_options, &option_index)) != -1) {
     switch (c) {
       case 0:	/* getopt_long options evaluation */
         break;
@@ -195,6 +198,10 @@ int main(int argc, char *argv[]) {
       case 'n':	/* pcapng format */
         pcapng++;
         break;
+      case 'o':	/* output file name */
+        filename_fix = malloc(strlen(optarg));
+	strcpy(filename_fix, optarg);
+        break;
       case 't':	/* data link type */
         data_link_type = atoi(optarg);
         break;
@@ -226,20 +233,27 @@ int main(int argc, char *argv[]) {
     return(-2);
   }
 
-  /* open output file */
-  /* we need to extract the basename first (windows and linux use different functions) */
-  filebname = malloc(strlen(filename));
-  #ifdef __WIN32__
-    _splitpath(filename, NULL, NULL, filebname, NULL);	/* windown method (_splitpath) */
-  # else
-    strcpy(filebname, basename(filename));		/* unix method (basename) */
-  #endif
-  filename_fix = malloc(strlen(filebname)+7);	/* size of outputfile depends on inputfile's length */
+  /* check for preassigned fixed file name */
+  if (filename_fix == NULL) {
 
-  /* prepare output file name: "fixed_" + inputfilename */
-  strcpy(filename_fix, "fixed_");
-  strcat(filename_fix, filebname);
-  free(filebname);
+    /* open output file */
+    /* we need to extract the basename first (windows and linux use different functions) */
+    filebname = malloc(strlen(filename)+1);
+    #ifdef __WIN32__
+      char *fileext = malloc(strlen(filename));   /* file extention to be used in output file as well */
+      _splitpath(filename, NULL, NULL, filebname, fileext);	/* windown method (_splitpath) */
+      strcat(filebname, fileext);
+      free(fileext);
+    # else
+      strcpy(filebname, basename(filename));		/* unix method (basename) */
+    #endif
+    filename_fix = malloc(strlen(filebname)+7);	/* size of outputfile depends on inputfile's length */
+
+    /* prepare output file name: "fixed_" + inputfilename */
+    strcpy(filename_fix, "fixed_");
+    strcat(filename_fix, filebname);
+    free(filebname);
+  }
 
   /* open the file for writing */
   pcap_fix = fopen(filename_fix, "wb");
@@ -322,6 +336,32 @@ int main(int argc, char *argv[]) {
 
       return(-6);
 
+    case BTSNOOP_MAGIC:
+      printf("[-] This is a BTSNOOP file, which is not supported.\n\n");
+
+      /* close input and output files */
+      fclose(pcap);
+      fclose(pcap_fix);
+
+      /* delete output file due to no changes failure */
+      remove(filename_fix);
+
+      return(-6);
+
+    case PCAP_EXT_MAGIC:
+    case PCAP_EXT_MAGIC_SWAPPED:
+      printf("[-] This is an extended tcpdump file, which is not supported, *YET*!\n");
+      printf("[*] If you need really need this filetype to be supported write a brief email to ruport@f00l.de\n\n");
+
+      /* close input and output files */
+      fclose(pcap);
+      fclose(pcap_fix);
+
+      /* delete output file due to no changes failure */
+      remove(filename_fix);
+
+      return(-6);
+
     /* PCAPNG format */
     case PCAPNG_MAGIC:
       printf("[+] This is a PCAPNG file.\n");
@@ -330,6 +370,7 @@ int main(int argc, char *argv[]) {
 
     /* classic PCAP format */
     case PCAP_MAGIC:
+    case PCAP_MAGIC_SWAPPED:
       printf("[+] This is a PCAP file.\n");
       if (pcapng > 0) {
         printf("[!] Your wish is my command! I will handle it as PCAPNG nevertheless.\n");
@@ -342,10 +383,10 @@ int main(int argc, char *argv[]) {
     /* if the file type is unknown (maybe header corrupted) assume classic PCAP format */
     default:
       if (pcapng > 0) {
-        printf("[*] Unknown filetype. Assuming PCAPNG format.\n");
+        printf("[*] Unknown file type. Assuming PCAPNG format.\n");
         res = fix_pcapng(pcap, pcap_fix);
       } else {
-        printf("[*] Unknown filetype. Assuming PCAP format.\n");
+        printf("[*] Unknown file type. Assuming PCAP format.\n");
         res = fix_pcap(pcap, pcap_fix);
       }
       break;
