@@ -123,6 +123,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
   unsigned int shb_num;                     /* number of SHB counter */
   unsigned int idb_num;                     /* number of IDB counter */
   unsigned int step;                        /* step counter for progress bar */
+  unsigned int packets;
 
   int64_t left;                             /* bytes left to proceed until current blocks end is reached */
   int fixes;                                /* corruptions counter */
@@ -135,13 +136,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
   /* init variables */
   pos = 0;            /* begin file check at position 0 */
+  packets = 0;
   fixes = 0;          /* no corruptions fixed yet */
   shb_num = 0;        /* no SHBs progressed yet */
   idb_num = 0;        /* no IDBs progressed yet */
   step = 1;           /* progress bar starts at 0 steps */
 
   /* loop every block inside pcapng file until end of file is reached */
-  while (pos < filesize) {
+
+  while (pos < filesize-sizeof(bh)) {
 
     /* print out progress bar if in non-verbose mode */
     if ((verbose == 0) && (5*(float)pos/(float)filesize > step)) {
@@ -207,7 +210,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
       /* Section Header Block */
       case TYPE_SHB:
-        if (verbose >= 1) printf("[*] FOUND: Section Header Block: 0x%08x (%u bytes)\n", bh.block_type, bh.total_length);
+        if (verbose >= 1) printf("[*] FOUND: Section Header Block at position %" PRIu64 " (%u bytes)\n", pos, bh.total_length);
 
         /* read section header block into struct */
         bytes = fread(&shb, sizeof(shb), 1, pcap);
@@ -246,10 +249,10 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
         /* section length */
         if (shb.section_length == -1) {
-          if (verbose >= 1) printf("[*] Section length: %" PRIu64 "\n", shb.section_length);
+          if (verbose >= 1) printf("[*] Section length: %" PRId64 "\n", shb.section_length);
 
         } else {
-          if (verbose >= 1) printf("[*] Section length: %" PRIu64 " ==> SETTING TO -1\n", shb.section_length);
+          if (verbose >= 1) printf("[*] Section length: %" PRId64 " ==> SETTING TO -1\n", shb.section_length);
           shb.section_length = -1;
         }
 
@@ -260,7 +263,6 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
         /* options */
         count = 0 ;
         while (left > 0) {
-
           /* read option header into struct */
           bytes = fread(&oh, sizeof(oh), 1, pcap);
           if (bytes != 1) return -3;
@@ -315,6 +317,33 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
           /* option is valid */
 
+          /* calculate padding for current option value */
+          padding = oh.option_length;
+          if (oh.option_length%4 != 0) padding += (4-oh.option_length%4);
+
+          /* check oversize */
+          if (padding > (unsigned)left) {
+            printf("[-] Option size (%" PRIu64 ") exceeds remaining block space (%" PRId64 "). ==> SKIPPING OPTION.\n", padding, left);
+            fixes++;
+
+            /* because this block oversizes, there should not be any further option */
+
+            /* is this the first option we check? */
+            if (count == 0) {
+              /* there are NO options inside this block; skipping EOO option */
+              if (verbose >= 1) printf("[*] No Options inside -> no need for End of Options...\n");
+              break;
+            }
+
+            /* there have been other options before this corruption, we need EOO option */
+
+            if (verbose >= 1) printf("[*] %u Options inside -> Finishing with End of Options...\n", count);
+
+            /* adjust option header to end of options */
+            oh.option_code = 0x00;
+            oh.option_length = 0x00;
+          }
+
           /* copy option header into repaired block */
           memcpy(new_block+block_pos, &oh, sizeof(oh));
           block_pos += sizeof(oh);
@@ -322,10 +351,6 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           /* end of options? -> do not write any further */
           if (oh.option_code == 0x00 && oh.option_length == 0x00) break;
 
-          /* calculate padding for current option value */
-          padding = oh.option_length;
-          if (oh.option_length%4 != 0) padding += (4-oh.option_length%4);
-
           /* read data of current option */
           data = malloc(padding);
           bytes = fread(data, padding, 1, pcap);
@@ -345,6 +370,15 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
       /* Packet Block */
       case TYPE_PB:
+	packets++;
+
+        /* check oversize */
+        if (sizeof(pb) > (unsigned)left) {
+          printf("[-] Packet #%u exceeds size of block header (%" PRIu64 " > %" PRId64 ") ==> SKIPPING.\n", packets, sizeof(pb), left);
+          /* set to "invalid block" */
+          bh.block_type = -1;
+          break;
+        }
 
         /* check for the mandatory SBH that MUST be before any packet! */
         if (shb_num == 0) {
@@ -357,16 +391,24 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           fixes++;
         }
 
-        if (verbose >= 2) printf("[*] FOUND: Packet Block: 0x%08x (%u bytes)\n", bh.block_type, bh.total_length);
+        if (verbose >= 1) printf("[*] FOUND Packet #%u: Packet Block at position %" PRIu64 " (%u bytes)\n", packets, pos, bh.total_length);
 
         /* read packet block into struct */
         bytes = fread(&pb, sizeof(pb), 1, pcap);
         if (bytes != 1) return -3;
         left -= sizeof(pb);
 
+        /* pre-check too large interface number (1024) */
+        if (pb.interface_id > 1024) {
+          /* interface id is unusal high --> this field is probably corrupted */
+          printf("[-] Probably corrupted Interface ID #%u (too high?) ==> CORRECTED.\n", pb.interface_id);
+          pb.interface_id = 1;
+          fixes++;
+        }
+
         /* check for the mandatory IDB that MUST identify every packets interface_id */
         while (pb.interface_id >= idb_num) {
-          /* no IDB identifying this packet, we need to create one - until the ID is reached */
+          /* no IDB is identifying this packet, we need to create one - until the ID has been reached */
           printf("[-] Missing IDB for Interface #%u ==> CREATING (#%u).\n", pb.interface_id, idb_num);
           write_idb(pcap_fix);
 
@@ -379,6 +421,12 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
         memcpy(new_block+block_pos, &pb, sizeof(pb));
         block_pos += sizeof(pb);
 
+        /* check for oversized caplen */
+        if (pb.caplen > (unsigned)left) {
+          printf("[-] Capture length (%u) exceeds block size (%" PRId64 ") ==> CORRECTED.\n", pb.caplen, left);
+          pb.caplen = left;
+        }
+
         /* calculate padding for packet data */
         padding = pb.caplen;
         if (pb.caplen % 4 != 0) padding += (4 - pb.caplen % 4);
@@ -449,6 +497,33 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
           /* option is valid */
 
+          /* calculate padding for current option value */
+          padding = oh.option_length;
+          if (oh.option_length%4 != 0) padding += (4-oh.option_length%4);
+
+          /* check oversize */
+          if (padding > (unsigned)left) {
+            printf("[-] Option size (%" PRIu64 ") exceeds remaining block space (%" PRId64 "). ==> SKIPPING OPTION.\n", padding, left);
+            fixes++;
+
+            /* because this block oversizes, there should not be any further option */
+
+            /* is this the first option we check? */
+            if (count == 0) {
+              /* there are NO options inside this block; skipping EOO option */
+              if (verbose >= 1) printf("[*] No Options inside -> no need for End of Options...\n");
+              break;
+            }
+
+            /* there have been other options before this corruption, we need EOO option */
+
+            if (verbose >= 1) printf("[*] %u Options inside -> Finishing with End of Options...\n", count);
+
+            /* adjust option header to end of options */
+            oh.option_code = 0x00;
+            oh.option_length = 0x00;
+          }
+
           /* copy options header into repaired block */
           memcpy(new_block+block_pos, &oh, sizeof(oh));
           block_pos += sizeof(oh);
@@ -456,10 +531,6 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           /* end of options? -> do not write any further */
           if (oh.option_code == 0x00 && oh.option_length == 0x00) break;
 
-          /* calculate padding for current option value */
-          padding = oh.option_length;
-          if (oh.option_length%4 != 0) padding += (4-oh.option_length%4);
-
           /* read data of current option */
           data = malloc(padding);
           bytes = fread(data, padding, 1, pcap);
@@ -478,6 +549,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
       /* Simple Packet Block */
       case TYPE_SPB:
+	packets++;
 
         /* check for the mandatory SBH that MUST be before any packet! */
         if (shb_num == 0) {
@@ -490,7 +562,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           fixes++;
         }
 
-        if (verbose >= 2) printf("[*] FOUND: Simple Packet Block: 0x%08x (%u bytes)\n", bh.block_type, bh.total_length);
+        if (verbose >= 1) printf("[*] FOUND Packet #%u: Simple Packet Block at position %" PRIu64 " (%u bytes)\n", packets, pos, bh.total_length);
 
         /* read simple packet block */
         bytes = fread(&spb, sizeof(spb), 1, pcap);
@@ -502,9 +574,10 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
         block_pos += sizeof(spb);
 
         /* calculate padding for packet data */
-        /* TODO: len is NOT the length of packet inside file (origlen != caplen); we need to calculate caplen using block length (left) */
-        padding = spb.len;
-        if (spb.len % 4 != 0) padding += (4 - spb.len % 4);
+        /* spb.len is NOT the length of packet inside file (origlen != caplen); we need to calculate caplen using block length (left) */
+        /* decrease by two to avoid oversize padding */
+        padding = left-2;
+        if (padding % 4 != 0) padding += (4 - left % 4);
 
         /* read packet data from input file */
         data = malloc(padding);
@@ -534,7 +607,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           fixes++;
         }
 
-        if (verbose >= 1) printf("[*] FOUND: Interface Description Block: 0x%08x (%u bytes)\n", bh.block_type, bh.total_length);
+        if (verbose >= 1) printf("[*] FOUND: Interface Description Block (%u bytes)\n", bh.total_length);
 
         /* read interface description block */
         bytes = fread(&idb, sizeof(idb), 1, pcap);	/* read first bytes of input file into struct */
@@ -643,6 +716,33 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
           /* option is valid */
 
+          /* calculate padding for current option value */
+          padding = oh.option_length;
+          if (oh.option_length%4 != 0) padding += (4-oh.option_length%4);
+
+          /* check oversize */
+          if (padding > (unsigned)left) {
+            printf("[-] Option size (%" PRIu64 ") exceeds remaining block space (%" PRId64 "). ==> SKIPPING OPTION.\n", padding, left);
+            fixes++;
+
+            /* because this block oversizes, there should not be any further option */
+
+            /* is this the first option we check? */
+            if (count == 0) {
+              /* there are NO options inside this block; skipping EOO option */
+              if (verbose >= 1) printf("[*] No Options inside -> no need for End of Options...\n");
+              break;
+            }
+
+            /* there have been other options before this corruption, we need EOO option */
+
+            if (verbose >= 1) printf("[*] %u Options inside -> Finishing with End of Options...\n", count);
+
+            /* adjust option header to end of options */
+            oh.option_code = 0x00;
+            oh.option_length = 0x00;
+          }
+
           /* copy options header into repaired block */
           memcpy(new_block+block_pos, &oh, sizeof(oh));
           block_pos += sizeof(oh);
@@ -650,10 +750,6 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           /* end of options? -> do not write any further*/
           if (oh.option_code == 0x00 && oh.option_length == 0x00) break;
 
-          /* calculate padding for current option value */
-          padding = oh.option_length;
-          if (oh.option_length%4 != 0) padding += (4-oh.option_length%4);
-
           /* read option data */
           data = malloc(padding);
           bytes = fread(data, padding, 1, pcap);
@@ -684,7 +780,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           fixes++;
         }
 
-        if (verbose >= 1) printf("[*] FOUND: Name Resolution Block: 0x%08x (%u bytes)\n", bh.block_type, bh.total_length);
+        if (verbose >= 1) printf("[*] FOUND: Name Resolution Block at position %" PRIu64 " (%u bytes)\n", pos, bh.total_length);
 
         /* process records */
         count = 0;
@@ -821,6 +917,33 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
           /* option is valid */
 
+          /* calculate padding for current option value */
+          padding = oh.option_length;
+          if (oh.option_length%4 != 0) padding += (4-oh.option_length%4);
+
+          /* check oversize */
+          if (padding > (unsigned)left) {
+            printf("[-] Option size (%" PRIu64 ") exceeds remaining block space (%" PRId64 "). ==> SKIPPING OPTION.\n", padding, left);
+            fixes++;
+
+            /* because this block oversizes, there should not be any further option */
+
+            /* is this the first option we check? */
+            if (count == 0) {
+              /* there are NO options inside this block; skipping EOO option */
+              if (verbose >= 1) printf("[*] No Options inside -> no need for End of Options...\n");
+              break;
+            }
+
+            /* there have been other options before this corruption, we need EOO option */
+
+            if (verbose >= 1) printf("[*] %u Options inside -> Finishing with End of Options...\n", count);
+
+            /* adjust option header to end of options */
+            oh.option_code = 0x00;
+            oh.option_length = 0x00;
+          }
+
           /* copy option header into repaired block */
           memcpy(new_block+block_pos, &oh, sizeof(oh));
           block_pos += sizeof(oh);
@@ -828,10 +951,6 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           /* end of options? -> do not write any further */
           if (oh.option_code == 0x00 && oh.option_length == 0x00) break;
 
-          /* calculate padding for current option value */
-          padding = oh.option_length;
-          if (oh.option_length%4 != 0) padding += (4-oh.option_length%4);
-
           /* read option value from input file */
           data = malloc(padding);
           bytes = fread(data, padding, 1, pcap);
@@ -863,7 +982,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           fixes++;
         }
 
-        if (verbose >= 1) printf("[*] FOUND: Interface Statistics Block: 0x%08x (%u bytes)\n", bh.block_type, bh.total_length);
+        if (verbose >= 1) printf("[*] FOUND: Interface Statistics Block at position %" PRIu64 " (%u bytes)\n", pos, bh.total_length);
 
         /* read interface statistics block */
         bytes = fread(&isb, sizeof(isb), 1, pcap);
@@ -948,6 +1067,33 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
           /* option is valid */
 
+          /* calculate padding for current option value */
+          padding = oh.option_length;
+          if (oh.option_length%4 != 0) padding += (4-oh.option_length%4);
+
+          /* check oversize */
+          if (padding > (unsigned)left) {
+            printf("[-] Option size (%" PRIu64 ") exceeds remaining block space (%" PRId64 "). ==> SKIPPING OPTION.\n", padding, left);
+            fixes++;
+
+            /* because this block oversizes, there should not be any further option */
+
+            /* is this the first option we check? */
+            if (count == 0) {
+              /* there are NO options inside this block; skipping EOO option */
+              if (verbose >= 1) printf("[*] No Options inside -> no need for End of Options...\n");
+              break;
+            }
+
+            /* there have been other options before this corruption, we need EOO option */
+
+            if (verbose >= 1) printf("[*] %u Options inside -> Finishing with End of Options...\n", count);
+
+            /* adjust option header to end of options */
+            oh.option_code = 0x00;
+            oh.option_length = 0x00;
+          }
+
           /* copy options header into repaired block */
           memcpy(new_block+block_pos, &oh, sizeof(oh));
           block_pos += sizeof(oh);
@@ -955,10 +1101,6 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           /* end of options? -> do not write any further */
           if (oh.option_code == 0x00 && oh.option_length == 0x00) break;
 
-          /* calculate padding for current option value */
-          padding = oh.option_length;
-          if (oh.option_length%4 != 0) padding += (4-oh.option_length%4);
-
           /* read option value from input file */
           data = malloc(padding);
           bytes = fread(data, padding, 1, pcap);
@@ -977,6 +1119,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
       /* Enhanced Packet Block */
       case TYPE_EPB:
+	packets++;
 
         /* check for the mandatory SBH that MUST be before any packet! */
         if (shb_num == 0) {
@@ -989,13 +1132,21 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           fixes++;
         }
 
-        if (verbose >= 2) printf("[*] FOUND: Enhanced Packet Block: 0x%08x (%u bytes)\n", bh.block_type, bh.total_length);
+        if (verbose >= 1) printf("[*] FOUND Packet #%u: Enhanced Packet Block at position %" PRIu64 " (%u bytes)\n", packets, pos, bh.total_length);
 
         /* read enhanced packet block */
         bytes = fread(&epb, sizeof(epb), 1, pcap);
         if (bytes != 1) return -3;
         left -= sizeof(epb);
 
+        /* pre-check too large interface number (1024) */
+        if (epb.interface_id > 1024) {
+          /* interface id is unusal high --> this field is probably corrupted */
+          printf("[-] Probably corrupted Interface ID #%u (too high?) ==> CORRECTED.\n", epb.interface_id);
+          epb.interface_id = 1;
+          fixes++;
+        }
+
         /* check for the mandatory IDB that MUST identify every packets interface_id */
         while (epb.interface_id >= idb_num) {
           /* no IDB identifying this packet, we need to create one - until the ID is reached */
@@ -1008,8 +1159,16 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
         }
 
         /* check if packet capture size exceeds packet length */
-        if ((int)epb.caplen > left) {
-          printf("[-] Enhanced packet data exceeds packet length (%u > %" PRIu64 ") ==> CORRECTED.\n", epb.caplen, left);
+        if (epb.caplen > epb.len) {
+          printf("[-] Enhanced packet data exceeds packet capture length (%u > %u) ==> CORRECTED.\n", epb.caplen, epb.len);
+          epb.caplen = epb.len;
+
+          fixes++;
+        }
+
+        /* check if packet capture size exceeds packet length */
+        if (epb.caplen > left) {
+          printf("[-] Enhanced packet data exceeds total packet size (%u > %" PRIu64 ") ==> CORRECTED.\n", epb.caplen, left);
           epb.caplen = left;
 
           fixes++;
@@ -1019,21 +1178,27 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
         memcpy(new_block+block_pos, &epb, sizeof(epb));
         block_pos += sizeof(epb);
 
-        /* calculate padding for packet data */
-        padding = epb.caplen;
-        if (epb.caplen % 4 != 0) padding += (4 - epb.caplen % 4);
+        /* check for zero capture length */
+        if (epb.caplen != 0) {
 
-        /* read packet data from input file */
-        data = malloc(padding);
-        bytes = fread(data, padding, 1, pcap);
-        left -= padding;
+          /* calculate padding for packet data */
+          padding = epb.caplen;
+          if (epb.caplen % 4 != 0) padding += (4 - epb.caplen % 4);
 
-        /* copy packet data into repaired block */
-        memcpy(new_block+block_pos, data, padding);
-        block_pos += padding;
+          /* read packet data from input file */
+          data = malloc(padding);
 
-        /* clean up memory */
-        free(data);
+          bytes = fread(data, padding, 1, pcap);
+          if (bytes != 1) return -3;
+          left -= padding;
+
+          /* copy packet data into repaired block */
+          memcpy(new_block+block_pos, data, padding);
+          block_pos += padding;
+
+          /* clean up memory */
+          free(data);
+        }
 
         /* options */
         count = 0;
@@ -1093,6 +1258,33 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
 
           /* option is valid */
 
+          /* calculate padding for current option value */
+          padding = oh.option_length;
+          if (oh.option_length%4 != 0) padding += (4-oh.option_length%4);
+
+          /* check oversize */
+          if (padding > (unsigned)left) {
+            printf("[-] Option size (%" PRIu64 ") exceeds remaining block space (%" PRId64 "). ==> SKIPPING OPTION.\n", padding, left);
+            fixes++;
+
+            /* because this block oversizes, there should not be any further option */
+
+            /* is this the first option we check? */
+            if (count == 0) {
+              /* there are NO options inside this block; skipping EOO option */
+              if (verbose >= 1) printf("[*] No Options inside -> no need for End of Options...\n");
+              break;
+            }
+
+            /* there have been other options before this corruption, we need EOO option */
+
+            if (verbose >= 1) printf("[*] %u Options inside -> Finishing with End of Options...\n", count);
+
+            /* adjust option header to end of options */
+            oh.option_code = 0x00;
+            oh.option_length = 0x00;
+          }
+
           /* copy option header into repaired block */
           memcpy(new_block+block_pos, &oh, sizeof(oh));
           block_pos += sizeof(oh);
@@ -1100,10 +1292,6 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
           /* end of options? -> do not write any further */
           if (oh.option_code == 0x00 && oh.option_length == 0x00) break;
 
-          /* calculate padding for current option value */
-          padding = oh.option_length;
-          if (oh.option_length%4 != 0) padding += (4-oh.option_length%4);
-
           /* read option value from input file */
           data = malloc(padding);
           bytes = fread(data, padding, 1, pcap);
@@ -1165,7 +1353,7 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
       if (verbose >= 2) printf("[+] End of Block reached... byte counter is correct!\n");
     } else {
       /* we did not read until end of block - maybe due to option skipping */
-      if (verbose >= 1) printf("[-] Did not hit the end of the block! (%" PRIu64 " bytes left)\n", left);
+      if (verbose >= 1) printf("[-] Did not hit the end of the block! (%" PRId64 " bytes left)\n", left);
     }
 
     /* check for correct block end (block size) */
@@ -1192,8 +1380,9 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
         if (verbose >= 1) printf("[*] Trying to align next block...\n");
         res = find_valid_block(pcap, filesize);
 
-        /* output information about skipped bytes */
-        printf("[-] Found %" PRIu64 " bytes of unknown data ==> SKIPPING.\n", ftello(pcap)-bytes);
+        /* output information about overlapped/skipped bytes */
+        if (ftello(pcap) > (unsigned)bytes) printf("[-] Found %" PRId64 " bytes of unknown data ==> SKIPPING.\n", ftello(pcap)-bytes);
+        else printf("[-] Packet overlapps with %" PRId64 " bytes ==> CORRECTED.\n", bytes-ftello(pcap));
 
         /* increase corruption counter */
         fixes++;
@@ -1241,6 +1430,8 @@ int find_valid_block(FILE *pcap, uint64_t filesize) {
   unsigned int bytes;
   unsigned int check;                       /* variable to check end of blocks sizes */
   struct block_header bh;
+  struct packet_block pb;                   /* Packet Block */
+  struct name_resolution_block nrb;         /* Name Resolution Block */
 
   /* bytewise processing of input file */
   for (i=ftello(pcap)-4; i<filesize; i++) {
@@ -1257,11 +1448,49 @@ int find_valid_block(FILE *pcap, uint64_t filesize) {
     if (bh.total_length >= 12 && bh.block_type >= TYPE_IDB && bh.block_type <= TYPE_EPB) {
       /* block header might be valid */
 
+      /* perform some block specific checks */
+
+      /* Packet Block Checks:
+       * - interface id <= 1024 */
+      if (bh.block_type == TYPE_PB) {
+        bytes = fread(&pb, sizeof(pb), 1, pcap);
+        if (bytes != 1) return(-1);
+
+        /* interface id check */
+        if (pb.interface_id > 1024) continue;
+      }
+
+      /* Simple Packet Block Checks:
+       * - max size <= 65535 */
+      if (bh.block_type == TYPE_SPB) {
+        /* max size check */
+        if (bh.total_length > 65536) continue;
+      }
+
+      /* Name Resolution Block Checks:
+       * - min size >= 16
+         - record size < block length
+         - record type ipv4,ipv6 or eeo */
+      if (bh.block_type == TYPE_NRB) {
+        bytes = fread(&nrb, sizeof(nrb), 1, pcap);
+        if (bytes != 1) return(-1);
+
+        /* max length check */
+        if (bh.total_length < 16) continue;
+
+        /* record length check */
+        if (nrb.record_length > bh.total_length) continue;
+
+        /* record type check (max is 0x02) */
+        if (nrb.record_type > 0x02) continue;
+      }
+
       /* check if the second size value is valid too */
       fseeko(pcap, i+bh.total_length-4, SEEK_SET);
       bytes = fread(&check, sizeof(check), 1, pcap);
       if (check == bh.total_length) {
         /* also the second block size value is correct! */
+
         if (verbose >= 1) printf("[+] FOUND: Block (Type: 0x%08x) at Position %" PRIu64 "\n", bh.block_type, i);
 
         /* set pointer to next block position */
